# Create your views here.
from StringIO import StringIO
import json
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import reverse
from django.db.models.loading import get_model
from django.utils.decorators import classonlymethod
from django.views.decorators.csrf import csrf_exempt
from django.http import HttpResponse, HttpResponseForbidden
from django.views.generic.base import View
from django.core.files.uploadedfile import InMemoryUploadedFile
import time

class CameraView(View):
    target = None
    target_field_name = None
    target_model = None
    target_dest_dir = None
    target_field_object = None

    @csrf_exempt
    def post(self, request, *args, **kwargs):
        op = request.POST.get('op', None)
        if op == 'save':
            return self._save(request, *args, **kwargs)
        elif op == 'rem':
            return self._delete(request, *args, **kwargs)
        else:
            return HttpResponseForbidden()

    @classonlymethod
    def as_view(cls, **initkwargs):
        target = initkwargs.get('target', '')
        bits = target.split('.')
        if len(bits) != 3:
            raise ImproperlyConfigured(
                'CameraView.as_view require a `target` argument in the format `app_label.model.field`')
        app_label, model_name, field_name = bits
        model = get_model(app_label, model_name)
        field_object, _, _, _= model._meta.get_field_by_name(field_name)

        initkwargs['target_model'] = model
        initkwargs['target_field_object'] = field_object
        initkwargs['target_field_name'] = field_name
        initkwargs['target_dest_dir'] = field_object.upload_to


        return super(CameraView, cls).as_view(**initkwargs)

    #ImageFieldFile
    def _delete(self, request, *args, **kwargs):
        try:
            target = self.target_model.objects.get(pk=self.kwargs['pk'])
            old_value = getattr(target, self.target_field_name)
            old_value.delete()
            target.save()
        except (IOError, OSError):
            pass
        return HttpResponse(json.dumps({
            'file': '',
            'url': '',
            'old': '',
            'refresh': "?%s" % time.time()
        }))


    def _save(self, request, *args, **kwargs):
        if self.request.POST['type'] == "pixel":
            pass
        elif self.request.POST['type'] == u"data":
            target = self.target_model.objects.get(pk=self.kwargs['pk'])
            raw_data = self.request.POST['image'][len('data:image/png;base64,'):].decode('base64')
            buf = StringIO(raw_data)
            buf.seek(0, 2)
            filename = "%s.png" % target.pk
            file = InMemoryUploadedFile(buf, "picture", filename, None, buf.tell(), None)
            old_picture = getattr(target, self.target_field_name)
            try:
                old_picture.delete()
            except OSError:
                pass
            setattr(target, 'picture', file)
            target.save()
            new_picture = getattr(target, self.target_field_name)
            full_url = reverse('camera', args=[new_picture.url])
        return HttpResponse(json.dumps({
            'file': filename,
            'url': full_url,
            'old': str(old_picture),
            'refresh': "%s?%s" % (full_url, time.time())
        }))



# -*- coding: utf-8 -*-
from django import forms
from django.forms.widgets import Widget
from django.template.loader import render_to_string, find_template


class CameraWidget(Widget):
    class Media:
        css = {
            'all': ('pretty.css',)
        }
        js = ('webcam/jquery.js',
              'webcam/jquery.webcam.js',
              'webcam/jquery.Jcrop.min.js',
              'webcam/camera.js')


    def render(self, name, value, attrs=None):
        return render_to_string('webcam/camera2.html', {'name':name, 'value':value, 'attrs':attrs})

class CameraField(forms.ImageField):
#    widget = CameraWidget

    def __init__(self, *args, **kwargs):
        super(CameraField, self).__init__(*args, **kwargs)




from django.db.models.fields.files import ImageField
from webcam import forms


class CameraField(ImageField):
    def __init__(self, verbose_name=None, name=None, width_field=None, height_field=None, **kwargs):
        super(CameraField, self).__init__(verbose_name, name, width_field, height_field, **kwargs)

    def formfield(self, **kwargs):
        defaults = {'form_class': forms.CameraField}
        defaults.update(kwargs)
        return super(CameraField, self).formfield(**defaults)



from django.core.exceptions import ImproperlyConfigured
from django.templatetags.static import static, PrefixNode
from urlparse import urljoin
from django import template
from django.utils.encoding import iri_to_uri

register = template.Library()

@register.inclusion_tag('webcam/camera.html')
def camera(save_url, initial=None, callback="_callback"):
    if not save_url:
        raise ImproperlyConfigured('`camera` templatetag expect a url as argument')

    return {'url': save_url,
            'callback':callback,
            'initial':initial,
            }

@register.filter
def default_image(imagefield, default):
    try:
        return imagefield.url
    except ValueError:
        return default



NAME = 'django-webcam'
VERSION = __version__ = (0, 1, 4, 'beta', 1)
__author__ = 'sax'


def get_version(version=None):
    """Derives a PEP386-compliant version number from VERSION."""
    if version is None:
        version = VERSION
    assert len(version) == 5
    assert version[3] in ('alpha', 'beta', 'rc', 'final')

    parts = 2 if version[2] == 0 else 3
    main = '.'.join(str(x) for x in version[:parts])

    sub = ''
    if version[3] == 'alpha' and version[4] == 0:
        import maintenance
        path =  maintenance.__path__[0]
        head_path = '%s/../.git/logs/HEAD' % path
        try:
            for line in open(head_path):pass
            revision = line.split()[0]
        except IOError:
            raise Exception('Alpha version is are only allowed as git clone')
        sub = '.dev%s' % revision

    elif version[3] != 'final':
        mapping = {'alpha': 'a', 'beta': 'b', 'rc': 'c'}
        sub = mapping[version[3]] + str(version[4])

    return main + sub




#!/usr/bin/env python
from distutils.core import setup
from distutils.command.install import INSTALL_SCHEMES
import os
import webcam

NAME = webcam.NAME
VERSION = RELEASE = webcam.get_version()

for scheme in INSTALL_SCHEMES.values():
    scheme['data'] = scheme['purelib']

packages, data_files = [], []
root_dir = os.path.dirname(__file__)
if root_dir != '':
    os.chdir(root_dir)



def fullsplit(path, result=None):
    """
    Split a pathname into components (the opposite of os.path.join) in a
    platform-neutral way.
    """
    if result is None:
        result = []
    head, tail = os.path.split(path)
    if head == '':
        return [tail] + result
    if head == path:
        return result
    return fullsplit(head, [tail] + result)


def scan_dir( target, packages=[], data_files=[] ):
    for dirpath, dirnames, filenames in os.walk(target):
        # Ignore dirnames that start with '.'
        for i, dirname in enumerate(dirnames):
            if dirname.startswith('.'): del dirnames[i]
        if '__init__.py' in filenames:
            packages.append('.'.join(fullsplit(dirpath)))
        elif filenames:
            data_files.append([dirpath, [os.path.join(dirpath, f) for f in filenames]])
    return packages, data_files

packages, data_files = scan_dir('webcam')

if VERSION[3] == 'final':
    download_tag = RELEASE
else:
    download_tag = 'master'

setup(
    name=NAME,
    version=RELEASE,
    url='https://github.com/saxix/django-webcam',
    download_url = 'https://github.com/saxix/django-webcam/tarball/%s' % download_tag,
    author='sax',
    author_email='sax@os4d.org',
    license='BSD',
    packages=packages,
    data_files=data_files,
    include_package_data=True,
    platforms=['any'],
    classifiers=[
#                 'Development Status :: 5 - Production/Stable',
                 'Environment :: Web Environment',
                 'Framework :: Django',
                 'Intended Audience :: Developers',
                 'License :: OSI Approved :: BSD License',
                 'Operating System :: OS Independent',
                 'Programming Language :: Python',
                 'Programming Language :: Python :: 2.5',
                 'Programming Language :: Python :: 2.6',
                 'Programming Language :: Python :: 2.7',
                 'Topic :: Internet :: WWW/HTTP',
                 'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
                 'Topic :: Internet :: WWW/HTTP :: WSGI',
                 'Topic :: Software Development :: Libraries :: Application Frameworks',
                 'Topic :: Software Development :: Libraries :: Python Modules',
                 ],
    long_description=open('README.rst').read()
)
